package flm.b4j.charts;

import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.objects.collections.List;
import anywheresoftware.b4j.objects.NodeWrapper;
import javafx.collections.ObservableList;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.chart.Chart;
import javafx.scene.chart.XYChart;
import javafx.scene.input.MouseEvent;

@BA.Hide
class XYCharts extends NodeWrapper<Chart>
{
	void Init(final BA ba, XYChart<?, ?> XYChart, String EventPrefix, String EventPrefixForPlot)
	{
		setObject(XYChart);
		Initialize(ba, EventPrefix);
		if (EventPrefixForPlot.length() > 0) {
			final String PrefixForPlot = EventPrefixForPlot.toLowerCase(BA.cul); 
			if (EventPrefix.length() > 0 && EventPrefix.toLowerCase(BA.cul).equals(PrefixForPlot)) {
				throw new IllegalArgumentException("EventPrefix must be different from EventPrefixForPlot.");
			}
			final String evtMouseEnteredPlot = PrefixForPlot + "_mouseentered";
			final String evtMouseMovedPlot = PrefixForPlot + "_mousemoved";
			final String evtMouseDraggedPlot = PrefixForPlot + "_mousedragged";
			final String evtMouseExitedPlot = PrefixForPlot + "_mouseexited";
			final String evtMousePressedPlot = PrefixForPlot + "_mousepressed";
			final String evtMouseReleasedPlot = PrefixForPlot + "_mousereleased";
			final Group PlotContent = (Group) getChartObject().lookup(".plot-content");
			if (ba.subExists(evtMouseEnteredPlot)) {
				PlotContent.setOnMouseEntered(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMouseEnteredPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
			if (ba.subExists(evtMouseMovedPlot)) {
				PlotContent.setOnMouseMoved(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMouseMovedPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
			if (ba.subExists(evtMouseDraggedPlot)) {
				PlotContent.setOnMouseDragged(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMouseDraggedPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
			if (ba.subExists(evtMouseExitedPlot)) {
				PlotContent.setOnMouseExited(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMouseExitedPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
			if (ba.subExists(evtMousePressedPlot)) {
				PlotContent.setOnMousePressed(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMousePressedPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
			if (ba.subExists(evtMouseReleasedPlot)) {
				PlotContent.setOnMouseReleased(new EventHandler<MouseEvent>() {
					@Override
					public void handle(MouseEvent event) {
						ba.raiseEvent(this, evtMouseReleasedPlot,
								new Object[] { AbsObjectWrapper.ConvertToWrapper(new MouseEventWrapper(), event) });
					}
				});
			}
		}
	}

	private XYChart<?, ?> getChartObject()
	{
		return (XYChart<?, ?>) getObject();
	}

	@SuppressWarnings("unchecked")
	public void AddSeries(b4jXYSeries XYSeries)
	{
		getChartObject().getData().add(XYSeries.getObject());
	}

	@SuppressWarnings("unchecked")
	public void RemoveSeries(b4jXYSeries XYSeries)
	{
		getChartObject().getData().removeAll(XYSeries.getObject());
	}

	/** Returns the list of all series currently displayed. */
	public List getSeries()
	{
		List L = new List();
		L.Initialize();
		ObservableList<?> AllData = getChartObject().getData();
		for(int i = 0; i < AllData.size(); i++)
			L.Add(AllData.get(i));
		return L;
	}

	public Object getXAxis()
	{
		return getChartObject().getXAxis();
	}
	public Object getYAxis()
	{
		return getChartObject().getYAxis();
	}

	/** Transforms the mouse position into axis coordinates. */
	public XYCoord MouseToAxis(MouseEventWrapper Event)
	{
		double SceneX = Event.getObject().getSceneX();
		double SceneY = Event.getObject().getSceneY();
		double xPosInAxis = ((Node) getXAxis()).sceneToLocal(SceneX, SceneY).getX();
		double yPosInAxis = ((Node) getYAxis()).sceneToLocal(SceneX, SceneY).getY();
		return new XYCoord(xPosInAxis, yPosInAxis);
	}

	/** Gets or sets whether the legend is visible (default = True). */
	public boolean getLegendVisible()
	{
		return getChartObject().isLegendVisible();
	}
	public void setLegendVisible(boolean Visible)
	{
		getChartObject().setLegendVisible(Visible);
	}

	/** Gets or sets whether the horizontal grid lines should be drawn (default = True). */
	public boolean getHorizontalGridLinesVisible()
	{
		return getChartObject().isHorizontalGridLinesVisible();
	}
	public void setHorizontalGridLinesVisible(boolean Visible)
	{
		getChartObject().setHorizontalGridLinesVisible(Visible);
	}

	/** Gets or sets whether the vertical grid lines should be drawn (default = True). */
	public boolean getVerticalGridLinesVisible()
	{
		return getChartObject().getVerticalGridLinesVisible();
	}
	public void setVerticalGridLinesVisible(boolean Visible)
	{
		getChartObject().setVerticalGridLinesVisible(Visible);
	}

	/** Gets or sets the chart title. */
	public String getTitle()
	{
		return getChartObject().getTitle();
	}
	public void setTitle(String Title)
	{
		getChartObject().setTitle(Title);
	}
}
